*********************************************************************
*                                                                   
*    	Diffractive parton distributions and structure functions      
*            	  		                                              
*                                                                   
*                                                                   
*       S.Taheri Monfared, A. N. Khorramian, S.Atashbar Tehrani
*                       arXiv:1109.0912                             
*                                                                   
*                                                                   
*                                                                   
*                                                                   
*            INPUT:                                                 
*                                                                   
*                     iset = 1 NEXT LEADING ORDER                   
*                     data file:  TKT.grid                           
*                                                                   
*            							                              
*         f2(2):      Structure function F2  (1=pomeron,2=reggeon)  
*         fl(2):      Structure function FL  (1=pomeron,2=reggeon)  
*         c2(2):      Structure function F2c (1=pomeron,2=reggeon)  
*         cl(2):      Structure function FLc (1=pomeron,2=reggeon)           
*         gl   :      gluon distribution                            
*         qua  :      quark distribution  	                      
*                                                                   
*             x  = beta      (between  0.0043 and 0.8)              
*             q2 = scale in GeV**2 (between  8.5 and 1600)          
*             H1-LRG-06   c=1.99*10**(-3.0)                         
*             H1-FPS-06   c=1.45*10**(-3.0)                         
*             H1-FPS-10   c=1.68*10**(-3.0)                         
*             ZEUS-LPS-04 c=1.92*10**(-3.0)                         
*             ZEUS-LPS-09 c=2.52*10**(-3.0)                         
*             ZEUS-LGR-09 c=3.15*10**(-3.0)                         
*             ZEUS-Mx-08  c=0                                       
*             ZEUS-Mx-05  c=0                                       
*                                                                   
* For upper bound of each value we use UP 						  
* For lower bound of each value we use DOWN                         
*********************************************************************
  
	  
      SUBROUTINE PPDF(ISET, X, Q2, F2p, F2Charm, FLp,
     1              FLCharm, f2_2, fl_2, c2_2, cl_2,qua, gl , 
     1quaUP, glUP, quaDOWN, glDOWN, 
     1F2pUP, FLpUP, F2pDOWN, FLpDOWN, 
	1 F2CharmUP, FLCharmUP, F2CharmDOWN, FLCharmDOWN)
*     ---------------------------------------------------------------

      IMPLICIT REAL*8 (A-H,O-Z)
*     
      PARAMETER (NPDF=22, NQ=30, NX=100 )
*
      
*
      COMMON / INTINI / IINI
*
      DIMENSION QS(NQ),XB(NX),XPDF(NX,NQ,NPDF)
      DIMENSION AXB(NX,NQ,NPDF),BXB(NX,NQ,NPDF),CXB(NX,NQ,NPDF)
      
*
      DIMENSION PDF1(NPDF),PDF2(NPDF)
      DIMENSION PDF(NPDF)
*
*
*
      SAVE QS, XB, XPDF
      SAVE AXB, BXB, CXB

*
*     CHECK OF X AND Q2 VALUES : 
*

      IF ( X.LT.0.0043 .OR. X.GT.0.8 ) THEN
         WRITE(6,2000) 
 2000    FORMAT (2X,'PARTON INTERPOLATION: X OUT OF RANGE -- STOP')
         STOP
      ENDIF
      IF ( Q2.LT.8.5 .OR. Q2.GT.1600 ) THEN
         WRITE(6,2001) 
 2001    FORMAT (2X,'PARTON INTERPOLATION: Q2 OUT OF RANGE -- STOP')
         STOP
      ENDIF
*
      IF (IINI .NE. 0) GOTO 10
*
*     READING OF THE DATA SET :
*     
      LIN = 10
*
      IF (ISET .EQ. 1) THEN
*
*     open input unit for LO data set N (Scenario 1)
*
         OPEN(UNIT=LIN,FILE='./TKT.grid',
     1              STATUS='UNKNOWN')
*

      ELSE
*
         WRITE(6,2003)
 2003    FORMAT (2X,'PARTON INTERPOLATION: ISET OUT OF RANGE -- '
     1        //'STOP')
         STOP
*
      ENDIF
*
      
      DO IQ = 1,NQ
         DO IX = 1,NX
            READ(LIN,*) QS(IQ),XB(IX),
     1           (XPDF(IX,IQ,IPDF),IPDF=1,NPDF)

         ENDDO
      ENDDO
*     
      CLOSE(LIN)
*
*     CALCULATE SPLINE COEFFIFIENTS FOR THE INTERPOLATION IN X :
*
      DO IPDF = 1,NPDF
         DO IQ = 1,NQ
            CALL SPLINE(NX,XB,XPDF,AXB,BXB,CXB,IPDF,IQ)
            
         ENDDO
      ENDDO
*
      IINI = 1
*
 10   CONTINUE
*
*     INTERPOLATION :
*     X: CUBIC SPLINE INTERPOLATION, LOG(Q2): LINEAR INTERPOLATION
*
      IQ = ISEARCH(NQ,QS,Q2)
      IF (IQ .EQ. NQ) IQ = NQ-1
      IX = ISEARCH(NX,XB,X)
*
      DX = X - XB(IX)
*
      DO IPDF = 1,NPDF
         PDF1(IPDF) = XPDF(IX,IQ,IPDF)
     1        + DX*(AXB(IX,IQ,IPDF) + DX*(BXB(IX,IQ,IPDF) 
     2        + DX*CXB(IX,IQ,IPDF)))
         PDF2(IPDF) = XPDF(IX,IQ+1,IPDF)
     1        + DX*(AXB(IX,IQ+1,IPDF) + DX*(BXB(IX,IQ+1,IPDF) 
     2        + DX*CXB(IX,IQ+1,IPDF)))
        
      ENDDO
*
      TQ = (DLOG(Q2)-DLOG(QS(IQ))) / (DLOG(QS(IQ+1))-DLOG(QS(IQ)))
*
      DO IPDF = 1,NPDF
         PDF(IPDF)  = (1.0D0-TQ)*PDF1(IPDF) + TQ*PDF2(IPDF)
         
      ENDDO
*
      F2p      = PDF(1)
      F2Charm  = PDF(2)  
      FLp      = PDF(3)
      FLCharm  = PDF(4) 
      f2_2     = PDF(5)
      fl_2     = PDF(6)
      c2_2     = PDF(7)
      cl_2     = PDF(8)
      qua      = PDF(9)
      gl       = PDF(10)
      quaUP    = PDF(11)
      glUP     = PDF(12)
	quaDOWN    = PDF(13)
      glDOWN   = PDF(14)
	F2pUP      = PDF(15)
	FLpUP      = PDF(16)
	F2pDOWN    = PDF(17)
	FLpDOWN    = PDF(18)
      F2CharmUP  = PDF(19)
	FLCharmUP  = PDF(20)
	F2CharmDOWN= PDF(21)
	FLCharmDOWN= PDF(22)
	
*
      RETURN
      END
*
* ---------------------------------------------------------------------
      SUBROUTINE SPLINE(N,X,Y,B,C,D,I,J)
* ---------------------------------------------------------------------
* CALCULATE THE COEFFICIENTS B,C,D IN A CUBIC SPLINE INTERPOLATION.
* INTERPOLATION SUBROUTINES ARE TAKEN FROM
* G.E. FORSYTHE, M.A. MALCOLM AND C.B. MOLER,
* COMPUTER METHODS FOR MATHEMATICAL COMPUTATIONS (PRENTICE-HALL, 1977).
*

*
      IMPLICIT REAL*8(A-H,O-Z)
*
      PARAMETER (NPDF=22, NQ=30, NX=100 )
*
      DIMENSION X(NX), Y(NX,NQ,NPDF),
     1          B(NX,NQ,NPDF), C(NX,NQ,NPDF), D(NX,NQ,NPDF)
*
      NM1=N-1
      IF(N.LT.2) RETURN
      IF(N.LT.3) GO TO 250
      D(1,J,I)=X(2)-X(1)
      C(2,J,I)=(Y(2,J,I)-Y(1,J,I))/D(1,J,I)
      DO 210 K=2,NM1
         D(K,J,I)=X(K+1)-X(K)
         B(K,J,I)=2.0D0*(D(K-1,J,I)+D(K,J,I))
         C(K+1,J,I)=(Y(K+1,J,I)-Y(K,J,I))/D(K,J,I)
         C(K,J,I)=C(K+1,J,I)-C(K,J,I)
  210 CONTINUE
      B(1,J,I)=-D(1,J,I)
      B(N,J,I)=-D(N-1,J,I)
      C(1,J,I)=0.0D0
      C(N,J,I)=0.0D0
      IF(N.EQ.3) GO TO 215
      C(1,J,I)=C(3,J,I)/(X(4)-X(2))-C(2,J,I)/(X(3)-X(1))
      C(N,J,I)=C(N-1,J,I)/(X(N)-X(N-2))-C(N-2,J,I)/(X(N-1)-X(N-3))
      C(1,J,I)=C(1,J,I)*D(1,J,I)**2.0D0/(X(4)-X(1))
      C(N,J,I)=-C(N,J,I)*D(N-1,J,I)**2.0D0/(X(N)-X(N-3))
 215  CONTINUE
      DO 220 K=2,N
         T=D(K-1,J,I)/B(K-1,J,I)
         B(K,J,I)=B(K,J,I)-T*D(K-1,J,I)
         C(K,J,I)=C(K,J,I)-T*C(K-1,J,I)
 220  CONTINUE
      C(N,J,I)=C(N,J,I)/B(N,J,I)
      DO 230 IB=1,NM1
         K=N-IB
         C(K,J,I)=(C(K,J,I)-D(K,J,I)*C(K+1,J,I))/B(K,J,I)
 230  CONTINUE
      B(N,J,I)=(Y(N,J,I)-Y(NM1,J,I))/D(NM1,J,I)
     1     +D(NM1,J,I)*(C(NM1,J,I)+2.0D0*C(N,J,I))
      DO 240 K=1,NM1
         B(K,J,I)=(Y(K+1,J,I)-Y(K,J,I))/D(K,J,I)
     1        -D(K,J,I)*(C(K+1,J,I)+2.0D0*C(K,J,I))
         D(K,J,I)=(C(K+1,J,I)-C(K,J,I))/D(K,J,I)
         C(K,J,I)=3.0D0*C(K,J,I)
 240  CONTINUE
      C(N,J,I)=3.0D0*C(N,J,I)
      D(N,J,I)=D(N-1,J,I)
      RETURN
 250  CONTINUE
      B(1,J,I)=(Y(2,J,I)-Y(1,J,I))/(X(2)-X(1))
      C(1,J,I)=0.0D0
      D(1,J,I)=0.0D0
      B(2,J,I)=B(1,J,I)
      C(2,J,I)=0.0D0
      D(2,J,I)=0.0D0
      RETURN
      END
*
* ---------------------------------------------------------------------
      INTEGER FUNCTION ISEARCH(N,X,Y)
* ---------------------------------------------------------------------
* THIS FUNCTION SEARCHES "I" WHICH SATISFIES THE RELATION
* X(I) <= Y < X(I+1) BY USING A BINARY SEARCH.
*

*
      IMPLICIT REAL*8(A-H,O-Z)
*
      PARAMETER (NPDF=22, NQ=30, NX=100 )
*
      DIMENSION X(NX)
*
      MIN=1
      MAX=N+1
*
   10 CONTINUE
      MID=(MIN+MAX)/2
      IF(Y.LT.X(MID)) THEN
        MAX=MID
      ELSE
        MIN=MID
      END IF
      IF((MAX-MIN).GT.1) GO TO 10
*
      ISEARCH=MIN
*
      RETURN
      END
*
* *********************************************************************
